; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.

;====== Flash commands =========================================================
;------ Flash write ------------------------------------------------------------
writeFlashByte:
; Writes a byte to flash.  Flash must be unlocked.  Be aware that pressing CLEAR
; will abort the write.
; Inputs:
;  - B: Byte to write
;  - C: Page
;  - HL: Address to write to.  This will not be wrapped.
; Outputs:
;  - NZ on failure
; Kills:
;  - AF, BC
; Protips:
;  - Calling writeFlashByteRaw instead will skip the page changing
;  - Do push bc \ pop af afterwards to restore flags to pre-call values.  That's
;    right, documented side-effect programming! 
	in	a, (6)	; save page
	push	af
	ld	a, c
	out	(6), a
	di
	call	writeFlashByteRaw
	ei
	pop	bc	; restore page without screwing up flags
	ld	a, b
	out	(6), a
	ret
writeFlashByteRaw:
; Flash program sequence
	ld	a, 0AAh	; First bus cycle---unlock
	ld	(0AAAh), a
	ld	a, 55h	; Second bus cycle---unlock
	ld	(0555h), a
	ld	a, 0A0h	; Third bus cycle---write command
	ld	(0AAAh), a
	ld	(hl), b	; Fourth bus cycle---program data
	; Wait for data to be good
	ld	a, 0FDh		; This checks for the CLEAR key.
	out	(1), a		; If pressed, it aborts.
	inc	hl
	dec	hl
programWaitLoop:
	in	a, (1)
	cp	0BFh
	jr	z, abortProgram
	ld	a, b
	xor	(hl)
	bit	7, a
	jr	z, programDone
	bit	5, (hl)
	jr	z, programWaitLoop
abortProgram:
	ld	a, 0F0h
	ld	(0000h), a
	inc	a
programDone:
	ret


;------ Erase sector -----------------------------------------------------------
eraseSector:
; Erases a sector of flash.
; Inputs:
;  - C: Page
; Outputs:
;  - Z on fail (this is opposite of writeFlashByte)
; Kills:
;  - AF, BC, HL
	in	a, (6)
	push	af
	ld	a, c
	out	(6), a
	ld	hl, 4000h
	di
	call	eraseSectorRaw
	ei
	pop	bc
	ld	a, b
	out	(6), a
	ret
eraseSectorRaw:
; Flash program sequence
	ld	a, 0AAh	; First bus cycle---unlock
	ld	(0AAAh), a
	ld	a, 55h	; Second bus cycle---unlock
	ld	(0555h), a
	ld	a, 080h	; Third bus cycle---write command
	ld	(0AAAh), a
	ld	a, 0AAh	; Fourth bus cycle---unlock (again)
	ld	(0AAAh), a
	ld	a, 55h	; Fifth bus cycle---unlock (again)
	ld	(0555h), a
	ld	a, 30h
	ld	(hl), a
	; Wait for data to be good
	ld	a, 0FDh
	out	(1), a
	inc	hl
	dec	hl        
eraseWaitLoop:
	in	a, (1)
	cp	0BFh
	jr	z, abortErase
	ld	a, (hl)
	bit	7, a
	jr	nz, eraseDone
	bit	5, a
	jr	z, eraseWaitLoop
abortErase:
	ld	a, 0F0h
	ld	(4000h), a
	xor	a
eraseDone:
	ret


;====== Math Routines ==========================================================
;------ Milos Bazelides routines -----------------------------------------------
; These routines are by Milos "baze" Bazelides, baze_at_baze_au_com
; Retrived from http://baze.au.com/misc/z80bits.html 25 October 2008

multHbyE:
; Input: H = Multiplier, E = Multiplicand
; Output: HL = Product
	xor	a
	ld	l, a
	ld	d, a
	sla	h		; optimised 1st iteration
	jr	nc,$+3
	ld	l,e
	
	ld	b, 7
multHbyEloop:
	add	hl,hl		; unroll 7 times
	jr	nc,$+3		; ...
	add	hl,de		; ...
	djnz	multHbyEloop
	ret


;====== Routines ===============================================================
PutS:
; Courtesy of Dan Englender
	ld	a, (hl)
	inc	hl
	or	a
	ret	z
	cp	chNewLine
	jr	z, PutSNewLine
	B_CALL(_PutC)
	jr	PutS
PutSNewLine:
	push	hl
	b_call(_NewLine)
	pop	hl
	jr	PutS


ldHLindirect:
	ld	a, (hl)
	inc	hl
	ld	h, (hl)
	ld	l, a	
	ret


cphlde:
	or	a
	sbc	hl,de
	add	hl,de
	ret


;------ ------------------------------------------------------------------------
dispByte:
; Displays a byte in hex.
; A is the byte to display.
	push	af
	push	bc
	push	af
	srl	a
	srl	a
	srl	a
	srl	a
	call	db1
	pop	af
	ld	b, 00001111b
	and	b
	call	db1
	pop	bc
	pop	af
	ret
db1:	ld	b, '0'
	cp	0Ah
	jr	c, db2
	ld	b, 'A'-0Ah
db2:	add	a, b
	B_CALL(_PutC)
	ret


;------ ------------------------------------------------------------------------
clearMem:
; Clears a section of memory SLOW! :p
; Input:
; - HL: Location to kill
; - B: Number of bytes to kill
; - A: What to clear it with
	ld	(hl), a
	inc	hl
	djnz	clearMem
	ret

	
;------ Menu -------------------------------------------------------------------
menu:
; Simple menu routine.
; HL points to a data structure that describes the menu.
; The first item in the menu is a zero-termed string to be displayed.
; After that, there is a list of acceptable keys and jump addresses.
; This is a JUMP not a CALL!
	im	1
	ei
	;push	hl	; extra push?
	;set	appAutoScroll, (iy+appFlags)
	push	hl
	B_CALL(_ClrScrn)
	ld	hl, 0
	ld	(curRow), hl
	pop	hl
	call	PutS ; hl = byte after 0
	push	hl
menuKeyLoop:
	b_call(_GetKey);_GetCSC)
	pop	hl
	push	hl
	ld	b, a
	ld	a, (hl) ; sanity check
	or	a ;
	jp	z, quit
_:	ld	a, (hl)
	or	a
	jr	z, menuKeyLoop
	cp	b
	jr	z, +_
	inc	hl
	inc	hl
	inc	hl
	jr	-_
_:	pop	de
	inc	hl
	call	ldHLindirect
	push	af
	ld	a, (hl)
	or	a
	jr	z, +_
	pop	af
	jp	(hl)
_:	pop	af
	inc	hl
	jp	menu


;------ Hex input --------------------------------------------------------------
getHexByte:
; Do not attempt to get a byte across a line break.
; Gets byte at current location.  Returns byte in A
; Registers:
; b: returns nibble from getNibble
; d: byte
	push	bc
	push	de
	push	hl
	B_CALL(_CursorOn)
getHexByteFirstNibble:
	res	secondNibble, (iy+inputerFlags)
	call	getNibble
	sla	b
	sla	b
	sla	b
	sla	b
	push	bc	;	ld	d, b
	set	secondNibble, (iy+inputerFlags)
	call	getNibble
	ld	a, b
	pop	bc
	add	a, b	;add	a, d
	push	af
	B_CALL(_CursorOff)
	pop	af
	pop	hl
	pop	de
	pop	bc
	ret
getNibble:
; Gets a single nibble.  Set secondNibble in IY+inputerFlags to allow backspace.
; Returns nibble in B.
	B_CALL(_GetCSC)
	ld	b, 0
	cp	sk0
	jr	z, nibble0
	cp	sk1
	jr	z, nibble1
	cp	sk2
	jr	z, nibble2
	cp	sk3
	jr	z, nibble3
	cp	sk4
	jr	z, nibble4
	cp	sk5
	jr	z, nibble5
	cp	sk6
	jr	z, nibble6
	cp	sk7
	jr	z, nibble7
	cp	sk8
	jr	z, nibble8
	cp	sk9
	jr	z, nibble9
	cp	skMath
	jr	z, nibbleA
	cp	skMatrix
	jr	z, nibbleB
	cp	skPrgm
	jr	z, nibbleC
	cp	skRecip
	jr	z, nibbleD
	cp	skSin
	jr	z, nibbleE
	cp	skCos
	jr	z, nibbleF
	cp	skDel
	jr	z, getNibbleBackspace
	cp	skLeft
	jr	z, getNibbleBackspace
;	cp	skClear
;	jp	z, quit
;	cp	skMode
;	jp	z, quit
	jr	getNibble
nibbleF: inc	b
nibbleE: inc	b
nibbleD: inc	b
nibbleC: inc	b
nibbleB: inc	b
nibbleA: inc	b
nibble9: inc	b
nibble8: inc	b
nibble7: inc	b
nibble6: inc	b
nibble5: inc	b
nibble4: inc	b
nibble3: inc	b
nibble2: inc	b
nibble1: inc	b
nibble0:
	ld	a, b
	ld	c, '0'
	ld	a, 9
	cp	b
	jr	nc, number
	ld	c, 'A'-0Ah
number:
	ld	a, b
	add	a, c
	B_CALL(_PutC)
	ret
getNibbleBackspace:
	bit	secondNibble, (iy+inputerFlags)
	jr	z, getNibble
	ld	a, (curCol)
	dec	a
	ld	(curCol), a
	ld	a, ' '
	B_CALL(_PutMap)
	pop	bc
	pop	bc				; stack
	jp	getHexByteFirstNibble

;------ ------------------------------------------------------------------------
; = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
getName:
; Gets a name for a program.  Does not allow any chars that are not legal in VAT
; names. . . mostly. :) 
; Input:
; - IX: where to dump the name.  Always zero-terms it.
; - (maxNameLength): the largest the name may be
; - enableLowercase, (iy+prgmFlags): set if you want to allow lowercase letters
; Output:
; - name in (ix)
; - ix points to 0-char
; Destroys:
; - All
; Requires tables.  These tables are easy to edit allow more characters.  Will
; not be happy if you attempt to backspace across a line wrap.
; Registers:
; de: pointer to end of table being scanned for keycodes
; hl: pointer to current item in table
; ix: pointer to destination of next char
; (iy+numberOfLetters): guess
; ShiftFlags: Holds status of modes
	ld	a, (CurCol)
	ld	(lastCurCol), a
	;push	ix
	b_call(_CursorOn)
	;pop	ix
	ld	a, (iy+ShiftFlags)
	and	%00000111
	ld	(iy+ShiftFlags), a
	set	ShiftAlpha, (iy+ShiftFlags)
	xor	a
	ld	(iy+numberOfLetters), a
getNameLoop:
	b_call(_GetCSC)
	cp	skEnter
	jr	z, imDoneWithYou
	cp	skClear
	jr	z, getRidOfAllOfIt
	cp	skMode
	jp	z, quit
	cp	skDel
	jp	z, flyYouFools
	cp	skLeft
	jp	z, flyYouFools
	cp	skAlpha
	jp	z, cycleAlpha
;	or	a
;	jr	z, getNameLoop
	bit	ShiftAlpha, (iy+ShiftFlags)
	jr	nz, PCLoadLetter
	ld	hl, numberTable
	ld	de, endNumberTable
	jr	searchKeyCodes
PCLoadLetter:
	ld	hl, letterTable
	ld	de, endLetterTable
searchKeyCodes:
	cp	(hl)
	jr	z, matchSticks
	inc	hl
	inc	hl
	call	cphlde
	jr	z, getNameLoop
	jr	searchKeyCodes
matchSticks:
	ld	a, (maxNameLength)
	cp	(iy+numberOfLetters)
	jr	z, getNameLoop	
	inc	hl
	ld	a, (hl)
	bit	ShiftLwrAlph, (iy+ShiftFlags)
	jr	nz, lowercaseAction
loadLetter:
	ld	(ix), a
	b_call(_PutC)
	inc	ix
	inc	(iy+numberOfLetters)
	jr	getNameLoop
lowercaseAction:
	cp	'[' ; theta
	jr	z, loadLetter
	add	a, $20
	jr	loadLetter
imDoneWithYou:
	res	ShiftAlpha, (iy+ShiftFlags)
	res	ShiftLwrAlph, (iy+ShiftFlags)
	xor	a
	cp	(iy+numberOfLetters)
	jr	z, getNameLoop
	ld	(ix), a
	inc	ix
	b_call(_CursorOff)
	ret
getRidOfAllOfIt:
	push	ix
	pop	hl
	ld	a, ' '
	b_call(_PutMap)
	ld	a, (iy+numberOfLetters)
	ld	e, a
	ld	d, 0
	or	a
	sbc	hl,de
	xor	a
	ld	(iy+numberOfLetters), a
	ld	a, (lastCurCol)
	ld	(CurCol), a
	ld	hl, spaces
	b_call(_PutS)
	ld	a, (lastCurCol)
	ld	(CurCol), a
	jp	getNameLoop
flyYouFools:
	xor	a
	cp	(iy+numberOfLetters)
	jp	z, getNameLoop
	ld	a, ' '
	b_call(_PutMap)
	dec	ix
	dec	(iy+numberOfLetters)
	ld	a, (CurCol)
	dec	a
	ld	(CurCol), a
	ld	a, ' '
	b_call(_PutMap)
	jp	getNameLoop
cycleAlpha:
	bit	ShiftAlpha, (iy+ShiftFlags)
	jr	z, setAlpha
	bit	enableLowercase, (iy+prgmFlags)
	jr	z, cycleToNumbers
	bit	ShiftLwrAlph, (iy+ShiftFlags)
	jr	z, setLwrAlpha
cycleToNumbers:
	res	ShiftAlpha, (iy+ShiftFlags)
	res	ShiftLwrAlph, (iy+ShiftFlags)
	jp	getNameLoop
setAlpha:
	set	ShiftAlpha, (iy+ShiftFlags)
	jp	getNameLoop
setLwrAlpha:
	set	ShiftLwrAlph, (iy+ShiftFlags)
	jp	getNameLoop
	
	
